@@ -1,3 +1,39 @@
+/************************************************************************
+
+	IMPORTANT NOTE : this file contains two clearly delimited sections :
+	the ARCHITECTURE section (in two parts) and the USER section. Each section
+	is governed by its own copyright and license. Please check individually
+	each section for license and copyright information.
+*************************************************************************/
+
+/*******************BEGIN ARCHITECTURE SECTION (part 1/2)****************/
+
+/************************************************************************
+    FAUST Architecture File
+	Copyright (C) 2003-2011 GRAME, Centre National de Creation Musicale
+    ---------------------------------------------------------------------
+    This Architecture section is free software; you can redistribute it
+    and/or modify it under the terms of the GNU General Public License
+	as published by the Free Software Foundation; either version 3 of
+	the License, or (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+	along with this program; If not, see <http://www.gnu.org/licenses/>.
+
+	EXCEPTION : As a special exception, you may create a larger work
+	that contains this FAUST architecture section and distribute
+	that work under terms of your choice, so long as this FAUST
+	architecture section is not modified.
+
+
+ ************************************************************************
+ ************************************************************************/
+
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
@@ -9,33 +45,27 @@
 #include <map>
 
 #include "ladspa.h"
+#include "gui/GUI.h"
+#include "misc.h"
+#include "audio/dsp.h"
 
-
-using namespace std;
-
-struct Meta : map<const char*, const char*>
-{
-    void declare (const char* key, const char* value) { (*this)[key]=value; }
-};
-
-	
-#define max(x,y) (((x)>(y)) ? (x) : (y))
-#define min(x,y) (((x)<(y)) ? (x) : (y))
+// On Intel set FZ (Flush to Zero) and DAZ (Denormals Are Zero)
+// flags to avoid costly denormals
+#ifdef __SSE__
+    #include <xmmintrin.h>
+    #ifdef __SSE2__
+        #define AVOIDDENORMALS _mm_setcsr(_mm_getcsr() | 0x8040)
+    #else
+        #define AVOIDDENORMALS _mm_setcsr(_mm_getcsr() | 0x8000)
+    #endif
+#else
+    #warning *** ladspa.cpp: NO SSE FLAG (denormals may slow things down) ***
+    #define AVOIDDENORMALS
+#endif
 
 #define sym(name) xsym(name)
 #define xsym(name) #name
 
-
-// abs is now predefined
-//template<typename T> T abs (T a)			{ return (a<T(0)) ? -a : a; }
-
-
-inline int		lsr (int x, int n)			{ return int(((unsigned int)x) >> n); }
-
-inline int 		int2pow2 (int x)	{ int r=0; while ((1<<r)<x) r++; return r; }
-
-
-
 /******************************************************************************
 *******************************************************************************
 
@@ -44,93 +74,17 @@ inline int 		int2pow2 (int x)	{ int r=0; while ((1<<r)<x) r++; return r; }
 *******************************************************************************
 *******************************************************************************/
 
-//inline void *aligned_calloc(size_t nmemb, size_t size) { return (void*)((unsigned)(calloc((nmemb*size)+15,sizeof(char)))+15 & 0xfffffff0); }
-//inline void *aligned_calloc(size_t nmemb, size_t size) { return (void*)((size_t)(calloc((nmemb*size)+15,sizeof(char)))+15 & ~15); }
-
-
 <<includeIntrinsic>>
 
+/********************END ARCHITECTURE SECTION (part 1/2)****************/
 
+/**************************BEGIN USER SECTION **************************/
 
-
-
-//---------------------Abstract User Interface--------------------
-//
-// Abstract definition of a User Interface to be passed to the 
-// buildUserInterface method of a Faust Signal Processor
-// 
-//----------------------------------------------------------------
-
-class UI
-{
-	bool	fStopped;
-public:
-		
-	UI() : fStopped(false) {}
-	virtual ~UI() {}
-	
-	virtual void addButton(const char* label, float* zone) = 0;
-	virtual void addToggleButton(const char* label, float* zone) = 0;
-	virtual void addCheckButton(const char* label, float* zone) = 0;
-	virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step) = 0;
-	virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step) = 0;
-	virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step) = 0;
-		
-	// -- passive widgets
-	
-	virtual void addNumDisplay(const char* label, float* zone, int precision) = 0;
-	virtual void addTextDisplay(const char* label, float* zone, char* names[], float min, float max) = 0;
-	virtual void addHorizontalBargraph(const char* label, float* zone, float min, float max) = 0;
-	virtual void addVerticalBargraph(const char* label, float* zone, float min, float max) = 0;
-
-	// -- frames and labels
-	
-	virtual void openFrameBox(const char* label) = 0;
-	virtual void openTabBox(const char* label) = 0;
-	virtual void openHorizontalBox(const char* label) = 0;
-	virtual void openVerticalBox(const char* label) = 0;
-	virtual void closeBox() = 0;
-	
-	virtual void show() = 0;
-	virtual void run() = 0;
-	
-	void stop()		{ fStopped = true; }
-	bool stopped() 	{ return fStopped; }
-
-    virtual void declare(float* zone, const char* key, const char* value) {}
-};
-
-
-
-//------------------Abstract Signal Processor---------------------
-//
-//  Abstract definition of a Faust Signal Processor
-//
-//----------------------------------------------------------------
-			
-class dsp {
- protected:
-	int fSamplingFreq;
- public:
-	dsp() {}
-	virtual ~dsp() {}
-	virtual int getNumInputs() 										= 0;
-	virtual int getNumOutputs() 									= 0;
-	virtual void buildUserInterface(UI* interface) 					= 0;
-	virtual void init(int samplingRate) 							= 0;
- 	virtual void compute(int len, float** inputs, float** outputs) 	= 0;
-};
-		
-
-//----------------------------------------------------------------------------
-// 	FAUST generated code
-//----------------------------------------------------------------------------
-		
 <<includeclass>>
 
+/***************************END USER SECTION ***************************/
 
-		
-	
+/*******************BEGIN ARCHITECTURE SECTION (part 2/2)***************/
 
 //-----------------------------------portCollector--------------------------------------
 //
@@ -140,8 +94,6 @@ class dsp {
 //
 //--------------------------------------------------------------------------------------
 
-
-
 //--------------------------------useful constants--------------------------------------
 
 #define MAXPORT 1024
@@ -171,10 +123,8 @@ static const char* onames[] = {
 					"output35", "output36", "output37", "output38", "output39"
 };
 
-
-
 class portCollector : public UI
-{		
+{
  private:
 
 	//--------------------------------------------------------------------------------------
@@ -182,101 +132,101 @@ class portCollector : public UI
 	const int				fInsCount;					// number of audio input ports
 	const int				fOutsCount;					// number of audio output ports
 	int						fCtrlCount;					// number of control ports
-	
+
 	LADSPA_PortDescriptor 	fPortDescs[MAXPORT];		// table of port descriptors to be used in a LADSPA_Descriptor
 	const char* 			fPortNames[MAXPORT];		// table of port names to be used in a LADSPA_Descriptor
 	LADSPA_PortRangeHint 	fPortHints[MAXPORT];		// table of port hints to be used in a LADSPA_Descriptor
-	
+
 	string					fPluginName;				// toplevel prefix used as plugin name
 	stack<string>			fPrefix;					// current prefix for controls name
 
-	
+
 	//--------------------------------------------------------------------------------------
 	string simplify(const string& src)
 	{
 		int		i=0;
 		int		level=2;
 		string	dst;
-		
+
 		while (src[i] ) {
-		
+
 			switch (level) {
-			
-				case 0 : 	
-				case 1 : 			
-				case 2 : 	
+
+				case 0 :
+				case 1 :
+				case 2 :
 					// Skip the begin of the label "--foo-"
 					// until 3 '-' have been read
 					if (src[i]=='-') { level++; }
 					break;
-							
-				case 3 :	
+
+				case 3 :
 					// copy the content, but skip non alphnum
 					// and content in parenthesis
 					switch (src[i]) {
-						case '(' : 	
-						case '[' : 	
+						case '(' :
+						case '[' :
 							level++;
 							break;
-							
-						case '-' : 	
+
+						case '-' :
 							dst += '-';
 							break;
-									
+
 						default :
 							if (isalnum(src[i])) {
-								dst+= tolower(src[i]); 
+								dst+= tolower(src[i]);
 							}
-							
+
 					}
 					break;
-					
-				default :	
-					// here we are inside parenthesis and 
+
+				default :
+					// here we are inside parenthesis and
 					// we skip the content until we are back to
 					// level 3
 					switch (src[i]) {
-		
-						case '(' : 	
-						case '[' : 
+
+						case '(' :
+						case '[' :
 							level++;
 							break;
-									
-						case ')' : 	
-						case ']' : 
+
+						case ')' :
+						case ']' :
 							level--;
 							break;
-							
+
 						default :
 							break;
 					}
-						
+
 			}
 			i++;
 		}
 		return (dst.size() > 0) ? dst :src;
 	}
 
-	void addPortDescr(int type, const char* label, int hint, float min=0.0, float max=0.0) 
+	void addPortDescr(int type, const char* label, int hint, float min=0.0, float max=0.0)
 	{
 		string fullname = simplify(fPrefix.top() + "-" + label);
 		char * str = strdup(fullname.c_str());
-		
-		fPortDescs[fInsCount + fOutsCount + fCtrlCount] = type; 
-		fPortNames[fInsCount + fOutsCount + fCtrlCount] = str; 
+
+		fPortDescs[fInsCount + fOutsCount + fCtrlCount] = type;
+		fPortNames[fInsCount + fOutsCount + fCtrlCount] = str;
 		fPortHints[fInsCount + fOutsCount + fCtrlCount].HintDescriptor = hint;
 		fPortHints[fInsCount + fOutsCount + fCtrlCount].LowerBound = min;
 		fPortHints[fInsCount + fOutsCount + fCtrlCount].UpperBound = max;
-		fCtrlCount++; 
+		fCtrlCount++;
 	}
-	
+
 	void openAnyBox(const char* label)
 	{
 		if (fPrefix.size() == 0) {
 			// top level label is used as plugin name
 			fPluginName = label;
 			fPrefix.push(label);
-			
+
 		} else {
 			string s;
 			if (label && label[0]) {
@@ -286,89 +236,81 @@ class portCollector : public UI
 			}
 			fPrefix.push(s);
 		}
-		
 	}
-	
 
-	
  public:
-		
-	
+
 	//--------------------------------Collect the audio ports-------------------------------
 
-	portCollector(int ins, int outs) : UI(), fInsCount(ins), fOutsCount(outs), fCtrlCount(0) 
+	portCollector(int ins, int outs) : UI(), fInsCount(ins), fOutsCount(outs), fCtrlCount(0)
 	{
-		for (int i = 0; i < ins; i++) { 
-			fPortDescs[i] = LADSPA_PORT_INPUT | LADSPA_PORT_AUDIO; 
-			fPortNames[i] = inames[i]; 
+		for (int i = 0; i < ins; i++) {
+			fPortDescs[i] = LADSPA_PORT_INPUT | LADSPA_PORT_AUDIO;
+			fPortNames[i] = inames[i];
 			fPortHints[i].HintDescriptor = 0;
 		}
 		for (int j = 0; j < outs; j++) {
-			fPortDescs[ins + j] = LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO; 
-			fPortNames[ins + j] = onames[j]; 
+			fPortDescs[ins + j] = LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO;
+			fPortNames[ins + j] = onames[j];
 			fPortHints[ins + j].HintDescriptor = 0;
 		}
 	};
 
 	virtual ~portCollector() {}
-	
-	
-	
+
 	//------------------------------Collect the control ports-------------------------------
 
-	virtual void addButton(const char* label, float* zone) { 
-		addPortDescr(ICONTROL, label, LADSPA_HINT_TOGGLED); 
-	}
-	
-	virtual void addToggleButton(const char* label, float* zone) { 
-		addPortDescr(ICONTROL, label, LADSPA_HINT_TOGGLED); 
-	}
-	
-	virtual void addCheckButton(const char* label, float* zone) { 
-		addPortDescr(ICONTROL, label, LADSPA_HINT_TOGGLED); 
-	}
-		
-	virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step) { 
-		addPortDescr(ICONTROL, label, LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, min, max); 
-	}		
-		
-	virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step) { 
-		addPortDescr(ICONTROL, label, LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, min, max); 
+	virtual void addButton(const char* label, float* zone) {
+		addPortDescr(ICONTROL, label, LADSPA_HINT_TOGGLED);
 	}
 
-	virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step) { 
-		addPortDescr(ICONTROL, label, LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, min, max); 
+	virtual void addToggleButton(const char* label, float* zone) {
+		addPortDescr(ICONTROL, label, LADSPA_HINT_TOGGLED);
 	}
-		
+
+	virtual void addCheckButton(const char* label, float* zone) {
+		addPortDescr(ICONTROL, label, LADSPA_HINT_TOGGLED);
+	}
+
+	virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step) {
+		addPortDescr(ICONTROL, label, LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, min, max);
+	}
+
+	virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step) {
+		addPortDescr(ICONTROL, label, LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, min, max);
+	}
+
+	virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step) {
+		addPortDescr(ICONTROL, label, LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, min, max);
+	}
+
 	// -- passive widgets
-	
+
 	virtual void addNumDisplay(const char* label, float* zone, int precision) {
-		addPortDescr(OCONTROL, label, 0, -10000, +10000); 
+		addPortDescr(OCONTROL, label, 0, -10000, +10000);
 	}
-	virtual void addTextDisplay(const char* label, float* zone, char* names[], float min, float max) {
-		addPortDescr(OCONTROL, label, LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, min, max); 
+	virtual void addTextDisplay(const char* label, float* zone, const char* names[], float min, float max) {
+		addPortDescr(OCONTROL, label, LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, min, max);
 	}
 	virtual void addHorizontalBargraph(const char* label, float* zone, float min, float max) {
-		addPortDescr(OCONTROL, label, LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, min, max); 
+		addPortDescr(OCONTROL, label, LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, min, max);
 	}
 	virtual void addVerticalBargraph(const char* label, float* zone, float min, float max){
-		addPortDescr(OCONTROL, label, LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, min, max); 
+		addPortDescr(OCONTROL, label, LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, min, max);
 	}
 
 	virtual void openFrameBox(const char* label)		{ openAnyBox(label); }
 	virtual void openTabBox(const char* label)		{ openAnyBox(label); }
 	virtual void openHorizontalBox(const char* label)	{ openAnyBox(label); }
 	virtual void openVerticalBox(const char* label)	{ openAnyBox(label); }
-	
+
 	virtual void closeBox() 					{ fPrefix.pop(); }
-	
+
 	virtual void show() {}
 	virtual void run() 	{}
-	
-	
-	
+
 	//---------------------------------Fill the LADSPA descriptor---------------------------
-	
+
 	// generate an ID from a plugin name
 	int makeID (const char* s) {
 		int h = 0;
@@ -385,7 +327,7 @@ class portCollector : public UI
 		descriptor->PortDescriptors 	= fPortDescs;
 		descriptor->PortNames 			= fPortNames;
 		descriptor->PortRangeHints 		= fPortHints;
-		
+
 		descriptor->Label = strdup(name);
 		descriptor->UniqueID = makeID(name);
 //		descriptor->Label = strdup(fPluginName.c_str());
@@ -398,85 +340,73 @@ class portCollector : public UI
 	}
 };
 
-
-
-
 //--------------------------------------portData----------------------------------------
 //
 // portData : a user interface used to associate the data buffers and the ports
 //
 //--------------------------------------------------------------------------------------
 
-
 class portData : public UI
-{	
+{
 
-	
  private:
 
-	
 	//--------------------------------------------------------------------------------------
 
 	const int				fInsCount;					// number of audio input ports
 	const int				fOutsCount;					// number of audio output ports
 	int						fCtrlCount;					// number of control ports
-	
-	float* 					fPortZone[MAXPORT];			// 
+
+	float* 					fPortZone[MAXPORT];			//
 	float* 					fPortData[MAXPORT];
-	
-	
+
 	//--------------------------------------------------------------------------------------
 
-	void addZone(float* zone) 
+	void addZone(float* zone)
 	{
-		fPortZone[fInsCount + fOutsCount + fCtrlCount] = zone; 
-		fCtrlCount++; 
+		fPortZone[fInsCount + fOutsCount + fCtrlCount] = zone;
+		fCtrlCount++;
 	}
-	
-	
-	
+
  public:
-		
-	
+
 	//--------------------------------Collect the audio ports-------------------------------
 
 	portData(int ins, int outs) : UI(), fInsCount(ins), fOutsCount(outs), fCtrlCount(0) {};
 	virtual ~portData() {}
-	
-	
+
 	//------------------------------Collect the control zones-------------------------------
 
 	virtual void addButton(const char* label, float* zone) 			{ addZone(zone); }
 	virtual void addToggleButton(const char* label, float* zone)  	{ addZone(zone); }
 	virtual void addCheckButton(const char* label, float* zone)  		{ addZone(zone); }
-		
+
 	virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step) 		{ addZone(zone); }
 	virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step) 	{ addZone(zone); }
 	virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step)  			{ addZone(zone); }
-		
+
 	// -- passive widgets
-	
+
 	virtual void addNumDisplay(const char* label, float* zone, int precision) 						{ addZone(zone); }
-	virtual void addTextDisplay(const char* label, float* zone, char* names[], float min, float max) 	{ addZone(zone); }
+	virtual void addTextDisplay(const char* label, float* zone, const char* names[], float min, float max) 	{ addZone(zone); }
 	virtual void addHorizontalBargraph(const char* label, float* zone, float min, float max) 			{ addZone(zone); }
 	virtual void addVerticalBargraph(const char* label, float* zone, float min, float max)			{ addZone(zone); }
-	
+
 	virtual void openFrameBox(const char* label)		{ }
 	virtual void openTabBox(const char* label)		{ }
 	virtual void openHorizontalBox(const char* label)	{ }
 	virtual void openVerticalBox(const char* label)	{ }
 	virtual void closeBox() 					{ }
-	
+
 	virtual void show() {}
 	virtual void run() 	{}
-	
-	
+
 	//---------------------------------interaction with LADSPA------------------------------
-	
+
 	void setPortData (unsigned long port, LADSPA_Data* data) {
 		fPortData[port] = data;
 	}
-	
+
 	void updateCtrlZones() {
 		for (int i = fInsCount+fOutsCount; i < fInsCount+fOutsCount+fCtrlCount; i++)	*fPortZone[i] = *fPortData[i];
 	}
@@ -487,12 +417,9 @@ class portData : public UI
 
 	float** getOutputs() {
 		return &fPortData[fInsCount];
-	}	
+	}
 };
 
-
-
-
 //--------------------------------Faust-LADSPA plugin-----------------------------------
 //
 // Plugin structure, callbacks and LADSPA_descriptor(i) entry point
@@ -501,25 +428,25 @@ class portData : public UI
 
 LADSPA_Descriptor* 	gDescriptor = 0;
 
-struct PLUGIN 
+struct PLUGIN
 {
 	unsigned long	fSampleRate;
 	portData*		fPortData;
 	dsp*			fDsp;
-	
+
 	PLUGIN(unsigned long r, portData* d, dsp* p) : fSampleRate(r), fPortData(d), fDsp(p) {}
 };
-	
-LADSPA_Handle instantiate_method (const struct _LADSPA_Descriptor * Descriptor, unsigned long SampleRate) 
+
+LADSPA_Handle instantiate_method (const struct _LADSPA_Descriptor * Descriptor, unsigned long SampleRate)
 {
 	dsp*		p = new mydsp();
 	portData* 	d = new portData(p->getNumInputs(), p->getNumOutputs());
-	
-	p->buildUserInterface(d);	
+
+	p->buildUserInterface(d);
 	return new PLUGIN (SampleRate, d, p);
 }
 
-void connect_method (LADSPA_Handle Instance, unsigned long Port, LADSPA_Data * DataLocation) 
+void connect_method (LADSPA_Handle Instance, unsigned long Port, LADSPA_Data * DataLocation)
 {
 	PLUGIN* p = (PLUGIN*) Instance;
 	p->fPortData->setPortData(Port, DataLocation);
@@ -535,12 +462,12 @@ void run_method (LADSPA_Handle Instance, unsigned long SampleCount)
 {
 	PLUGIN* p = (PLUGIN*) Instance;
 	p->fPortData->updateCtrlZones();
+	AVOIDDENORMALS;
 	p->fDsp->compute(SampleCount, p->fPortData->getInputs(), p->fPortData->getOutputs());
 }
 
 void deactivate_method (LADSPA_Handle Instance)
-{
-}
+{}
 
 void cleanup_method (LADSPA_Handle Instance)
 {
@@ -550,10 +477,9 @@ void cleanup_method (LADSPA_Handle Instance)
 	delete p;
 }
 
-
 //--------------------------------------------------------------------------------------
 
-void init_descriptor(LADSPA_Descriptor* descriptor) 
+void init_descriptor(LADSPA_Descriptor* descriptor)
 {
 	descriptor->UniqueID = 123456;
 	descriptor->Label = "none";
@@ -561,9 +487,9 @@ void init_descriptor(LADSPA_Descriptor* descriptor)
 	descriptor->Name = "none";
 	descriptor->Maker = "Yann Orlarey";
 	descriptor->Copyright = "GPL";
-	
+
 	descriptor->ImplementationData = 0;
-	
+
 	// description des methods
 	descriptor->instantiate = instantiate_method;
 	descriptor->connect_port = connect_method;
@@ -575,14 +501,12 @@ void init_descriptor(LADSPA_Descriptor* descriptor)
 	descriptor->cleanup = cleanup_method;
 }
 
-
-
 //--------------------------------------------------------------------------------------
 
-const LADSPA_Descriptor * ladspa_descriptor(unsigned long Index) 
+const LADSPA_Descriptor * ladspa_descriptor(unsigned long Index)
 {
     if (Index == 0) {
-		if (gDescriptor == 0) 
+		if (gDescriptor == 0)
 		{
 			// allocate temporaries dsp and portCollector to build the plugin description
 			mydsp* p = new mydsp();
@@ -603,3 +527,6 @@ const LADSPA_Descriptor * ladspa_descriptor(unsigned long Index)
 	}
 }
 
+/********************END ARCHITECTURE SECTION (part 2/2)****************/
+
+
