@@ -42,9 +42,15 @@ inline int 		int2pow2 (int x)		{ int r=0; while ((1<<r)<x) r++; return r; }
 long lopt(char *argv[], const char *name, long def)
 {
 	int	i;
-	for (i=0; argv[i]; i++) if (!strcmp(argv[i], name)) return atoi(argv[i+1]);
+	for (i = 0; argv[i]; i++) if (!strcmp(argv[i], name)) return atoi(argv[i+1]);
 	return def;
 }
 
+char* lopts(char *argv[], const char *name, char* def)
+{
+	int	i;
+	for (i = 0; argv[i]; i++) if (!strcmp(argv[i], name)) return argv[i+1];
+	return def;
+}
 #endif
 
