@@ -1,3 +1,39 @@
+/************************************************************************
+
+	IMPORTANT NOTE : this file contains two clearly delimited sections : 
+	the ARCHITECTURE section (in two parts) and the USER section. Each section 
+	is governed by its own copyright and license. Please check individually 
+	each section for license and copyright information.
+*************************************************************************/
+
+/*******************BEGIN ARCHITECTURE SECTION (part 1/2)****************/
+
+/************************************************************************
+    FAUST Architecture File
+	Copyright (C) 2003-2011 GRAME, Centre National de Creation Musicale
+    ---------------------------------------------------------------------
+    This Architecture section is free software; you can redistribute it 
+    and/or modify it under the terms of the GNU General Public License 
+	as published by the Free Software Foundation; either version 3 of 
+	the License, or (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License 
+	along with this program; If not, see <http://www.gnu.org/licenses/>.
+
+	EXCEPTION : As a special exception, you may create a larger work 
+	that contains this FAUST architecture section and distribute  
+	that work under terms of your choice, so long as this FAUST 
+	architecture section is not modified. 
+
+
+ ************************************************************************
+ ************************************************************************/
+
 /* link with  */
 #include <stdlib.h>
 #include <stdio.h>
@@ -505,14 +541,16 @@ GTKUI::GTKUI(char * name, int* pargc, char*** pargv)
 
 void GTKUI::pushBox(int mode, GtkWidget* w)
 {
-	assert(++fTop < stackSize);
+	++fTop;
+	assert(fTop < stackSize);
 	fMode[fTop] 	= mode;
 	fBox[fTop] 		= w;
 }
 
 void GTKUI::closeBox()
 {
-	assert(--fTop >= 0);
+	--fTop;
+	assert(fTop >= 0);
 }
 
 
@@ -963,13 +1001,15 @@ class dsp {
  	virtual void compute(int len, float** inputs, float** outputs) 	= 0;
 };
 		
+/********************END ARCHITECTURE SECTION (part 1/2)****************/
 
-//----------------------------------------------------------------------------
-// 	FAUST generated code
-//----------------------------------------------------------------------------
+/**************************BEGIN USER SECTION **************************/
 		
 <<includeclass>>
-		
+
+/***************************END USER SECTION ***************************/
+
+/*******************BEGIN ARCHITECTURE SECTION (part 2/2)***************/
 				
 mydsp	DSP;
 
@@ -1137,4 +1177,5 @@ int main(int argc, char *argv[] )
     return 0;
 }
 
+/********************END ARCHITECTURE SECTION (part 2/2)****************/
 
