@@ -1,12 +1,12 @@
 /************************************************************************
  ************************************************************************
     FAUST Architecture File
-	Copyright (C) 2009-2011 Albert Graef <Dr.Graef@t-online.de>
+    Copyright (C) 2009-2011 Albert Graef <Dr.Graef@t-online.de>
     ---------------------------------------------------------------------
     This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU Lesser General Public License as 
-	published by the Free Software Foundation; either version 2.1 of the 
-	License, or (at your option) any later version.
+    it under the terms of the GNU Lesser General Public License as
+    published by the Free Software Foundation; either version 2.1 of the
+    License, or (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,9 +14,9 @@
     GNU Lesser General Public License for more details.
 
     You should have received a copy of the GNU Lesser General Public
- 	License along with the GNU C Library; if not, write to the Free
-  	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-  	02111-1307 USA. 
+    License along with the GNU C Library; if not, write to the Free
+    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+    02111-1307 USA.
  ************************************************************************
  ************************************************************************/
 
@@ -25,21 +25,28 @@
    http://pure-lang.googlecode.com for a Pure module which can load these
    extensions. */
 
-
 #include <stdlib.h>
 #include <math.h>
+#include <list>
+#include <map>
+
+using namespace std;
+
+typedef pair<const char*,const char*> strpair;
 
 struct Meta
 {
-    void declare (const char* key, const char* value) { }
+  list< strpair > data;
+  void declare (const char* key, const char* value)
+  { data.push_back(strpair(key, value)); }
 };
 
 //-------------------------------------------------------------------
 // Generic min and max using c++ inline
 //-------------------------------------------------------------------
 
-inline int 		max (unsigned int a, unsigned int b) { return (a>b) ? a : b; }
-inline int 		max (int a, int b) 			{ return (a>b) ? a : b; }
+inline int 	max (unsigned int a, unsigned int b) { return (a>b) ? a : b; }
+inline int 	max (int a, int b)		{ return (a>b) ? a : b; }
 
 inline long 	max (long a, long b) 		{ return (a>b) ? a : b; }
 inline long 	max (int a, long b) 		{ return (a>b) ? a : b; }
@@ -60,7 +67,7 @@ inline double 	max (float a, double b) 	{ return (a>b) ? a : b; }
 inline double 	max (double a, float b) 	{ return (a>b) ? a : b; }
 
 
-inline int 		min (int a, int b) 			{ return (a<b) ? a : b; }
+inline int	min (int a, int b)		{ return (a<b) ? a : b; }
 
 inline long 	min (long a, long b) 		{ return (a<b) ? a : b; }
 inline long 	min (int a, long b) 		{ return (a<b) ? a : b; }
@@ -81,10 +88,9 @@ inline double 	min (float a, double b) 	{ return (a<b) ? a : b; }
 inline double 	min (double a, float b) 	{ return (a<b) ? a : b; }
 
 // abs is now predefined
-//template<typename T> T abs (T a)			{ return (a<T(0)) ? -a : a; }
+//template<typename T> T abs (T a)		{ return (a<T(0)) ? -a : a; }
 
-
-inline int		lsr (int x, int n)			{ return int(((unsigned int)x) >> n); }
+inline int	lsr (int x, int n)		{ return int(((unsigned int)x) >> n); }
 
 /******************************************************************************
 *******************************************************************************
@@ -111,10 +117,10 @@ class UI
 {
   bool	fStopped;
 public:
-		
+
   UI() : fStopped(false) {}
   virtual ~UI() {}
-	
+
   virtual void addButton(const char* label, double* zone) = 0;
   virtual void addToggleButton(const char* label, double* zone) = 0;
   virtual void addCheckButton(const char* label, double* zone) = 0;
@@ -126,17 +132,17 @@ public:
   virtual void addTextDisplay(const char* label, double* zone, char* names[], float min, float max) = 0;
   virtual void addHorizontalBargraph(const char* label, double* zone, float min, float max) = 0;
   virtual void addVerticalBargraph(const char* label, double* zone, float min, float max) = 0;
-	
+
   virtual void openFrameBox(const char* label) = 0;
   virtual void openTabBox(const char* label) = 0;
   virtual void openHorizontalBox(const char* label) = 0;
   virtual void openVerticalBox(const char* label) = 0;
   virtual void closeBox() = 0;
-	
+
   virtual void run() = 0;
-	
-  void stop()	{ fStopped = true; }
-  bool stopped() 	{ return fStopped; }
+
+  void stop() { fStopped = true; }
+  bool stopped() { return fStopped; }
 
   virtual void declare(double* zone, const char* key, const char* value) {}
 };
@@ -165,7 +171,8 @@ class PureUI : public UI
 public:
   int nelems;
   ui_elem_t *elems;
-		
+  map< int, list<strpair> > metadata;
+
   PureUI();
   virtual ~PureUI();
 
@@ -189,14 +196,16 @@ public:
   virtual void addTextDisplay(const char* label, double* zone, char* names[], float min, float max);
   virtual void addHorizontalBargraph(const char* label, double* zone, float min, float max);
   virtual void addVerticalBargraph(const char* label, double* zone, float min, float max);
-  
+
   virtual void openFrameBox(const char* label);
   virtual void openTabBox(const char* label);
   virtual void openHorizontalBox(const char* label);
   virtual void openVerticalBox(const char* label);
   virtual void closeBox();
-	
+
   virtual void run();
+
+  virtual void declare(double* zone, const char* key, const char* value);
 };
 
 PureUI::PureUI()
@@ -210,6 +219,15 @@ PureUI::~PureUI()
   if (elems) free(elems);
 }
 
+void PureUI::declare(double* zone, const char* key, const char* value)
+{
+  map< int, list<strpair> >::iterator it = metadata.find(nelems);
+  if (it != metadata.end())
+    it->second.push_back(strpair(key, value));
+  else
+    metadata[nelems] = list<strpair>(1, strpair(key, value));
+}
+
 inline void PureUI::add_elem(ui_elem_type_t type, const char *label)
 {
   ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
@@ -325,23 +343,23 @@ void PureUI::run() {}
 *******************************************************************************
 *******************************************************************************/
 
-
-
 //----------------------------------------------------------------
 //  abstract definition of a signal processor
 //----------------------------------------------------------------
-			
+
 class dsp {
  protected:
-	int fSamplingFreq;
+  int fSamplingFreq;
  public:
-	dsp() {}
-	virtual ~dsp() {}
-	virtual int getNumInputs() = 0;
-	virtual int getNumOutputs() = 0;
-	virtual void buildUserInterface(UI* interface) = 0;
-	virtual void init(int samplingRate) = 0;
- 	virtual void compute(int len, double** inputs, double** outputs) = 0;
+  // internal freelist for custom voice allocation
+  dsp *prev, *next;
+  dsp() {}
+  virtual ~dsp() {}
+  virtual int getNumInputs() = 0;
+  virtual int getNumOutputs() = 0;
+  virtual void buildUserInterface(UI* interface) = 0;
+  virtual void init(int samplingRate) = 0;
+  virtual void compute(int len, double** inputs, double** outputs) = 0;
 };
 
 //----------------------------------------------------------------------------
@@ -356,16 +374,145 @@ class dsp {
 
 <<includeclass>>
 
-/* The class factory, used to create and destroy mydsp objects in the
-   client. Implemented using C linkage to facilitate dlopen access. */
+#include <assert.h>
 
-extern "C" dsp *newdsp()
+// Define this to get some debugging output.
+//#define DEBUG
+#ifdef DEBUG
+#include <stdio.h>
+#define FAUST_CN "mydsp"
+#endif
+
+/* Dynamic voice allocation. We go to some lengths here to make this as
+   realtime-friendly as possible. The idea is that we keep a pool of allocated
+   mydsp instances. When a dsp is freed with deldsp(), it's in fact never
+   deleted, but put at the end of a freelist from where it may eventually be
+   reused by a subsequent call to newdsp(). By these means, the number of
+   actual calls to malloc() can be kept to a minimum. In addition, a small
+   number of voices are preallocated in static memory (1 by default in the
+   present implementation, but you can set this at compile time by redefining
+   the NVOICES constant accordingly). If you choose a suitable NVOICES value,
+   chances are that your application may never need to allocate dsp instances
+   on the heap at all. Also, even if dsp instances have to be created
+   dynamically, they are allocated in chunks of NVOICES units, in order to
+   reduce the calls to malloc(). Thus we generally recommend to set NVOICES to
+   a value >1 which best suits your application. */
+
+#ifndef NVOICES
+#define NVOICES 1
+#endif
+
+// Make sure that NVOICES is at least 1.
+#if NVOICES<1
+#undefine NVOICES
+#define NVOICES 1
+#endif
+
+struct dspmem_t {
+  char x[sizeof(mydsp)];
+};
+
+struct mem_t {
+  dspmem_t mem[NVOICES];
+  mem_t *next;
+};
+
+// statically and dynamically allocated dsp instances
+static mem_t mem0, *mem;
+// beginning and end of the freelist
+static mydsp *first, *last;
+
+/* This is supposed to be executed when the module gets unloaded. You'll need
+   a recent gcc version (or compatible) to make this work. */
+
+void __attribute__ ((destructor)) mydsp_fini(void)
 {
-  mydsp *d = new mydsp();
+  if (!mem) return;
+  mem = mem->next;
+  while (mem) {
+    mem_t *mem1 = mem->next;
+    free(mem); mem = mem1;
+  }
+}
+
+/* The class factory, used to create and destroy mydsp objects in the client.
+   This is implemented using C linkage to facilitate dlopen access. */
+
+#include <new>
+
+extern "C" mydsp *newdsp()
+{
+  if (!mem) {
+    mem = &mem0; mem->next = 0;
+    // initialize the freelist with the statically allocated voices
+    mydsp *prev = 0, *next = (mydsp*)&mem->mem[0];
+    first = next;
+    for (int i = 0; i < NVOICES; i++) {
+      void *p = &mem->mem[i];
+      mydsp *d = new(p) mydsp;
+      d->prev = prev; prev = d;
+      d->next = ++next;
+    }
+    last = prev; last->next = 0;
+#ifdef DEBUG
+    fprintf(stderr, ">>> %s: preallocated %d voices\n", FAUST_CN, NVOICES);
+#endif
+  }
+  assert(mem);
+  if (!first) {
+    // allocate a new chunk of voices and add them to the freelist
+    mem_t *block = (mem_t*)calloc(1, sizeof(mem_t));
+    block->next = mem->next; mem->next = block;
+    mydsp *prev = 0, *next = (mydsp*)&block->mem[0];
+    first = next;
+    for (int i = 0; i < NVOICES; i++) {
+      void *p = &block->mem[i];;
+      mydsp *d = new(p) mydsp;
+      d->prev = prev; prev = d;
+      d->next = ++next;
+    }
+    last = prev; last->next = 0;
+#ifdef DEBUG
+    fprintf(stderr, ">>> %s: allocated %d voices\n", FAUST_CN, NVOICES);
+#endif
+  }
+  assert(first && last);
+  mydsp *d = first;
+  if (first == last) {
+    // freelist is now empty
+    first = last = 0;
+  } else {
+    // remove d from the freelist
+    first = (mydsp*)first->next;
+  }
+  d->prev = d->next = 0;
+#ifdef DEBUG
+  fprintf(stderr, ">>> %s: allocating instance %p\n", FAUST_CN, d);
+#endif
   return d;
 }
 
-extern "C" void deldsp(dsp* d)
+extern "C" void deldsp(mydsp* d)
 {
-  delete d;
+#ifdef DEBUG
+  fprintf(stderr, ">>> %s: freeing instance %p\n", FAUST_CN, d);
+#endif
+  // add d to the freelist
+  assert(!d->prev && !d->next);
+  if (last) {
+    last->next = d; d->prev = last; last = d;
+  } else
+    first = last = d;
+}
+
+extern "C" Meta *newmeta()
+{
+  Meta *m = new Meta;
+  mydsp::metadata(m);
+  return m;
+}
+
+extern "C" void delmeta(Meta* m)
+{
+  delete m;
 }
