@@ -1,3 +1,25 @@
+/************************************************************************
+ ************************************************************************
+    FAUST Architecture File
+	Copyright (C) 2006-2011 Albert Graef <Dr.Graef@t-online.de>
+    ---------------------------------------------------------------------
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU Lesser General Public License as 
+	published by the Free Software Foundation; either version 2.1 of the 
+	License, or (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+ 	License along with the GNU C Library; if not, write to the Free
+  	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+  	02111-1307 USA. 
+ ************************************************************************
+ ************************************************************************/
+
 /* Pd architecture file, written by Albert Graef <Dr.Graef@t-online.de>.
    This was derived from minimal.cpp included in the Faust distribution.
    Please note that this is to be compiled as a shared library, which is
@@ -10,6 +32,20 @@
 
 using namespace std;
 
+// On Intel set FZ (Flush to Zero) and DAZ (Denormals Are Zero)
+// flags to avoid costly denormals
+#ifdef __SSE__
+    #include <xmmintrin.h>
+    #ifdef __SSE2__
+        #define AVOIDDENORMALS _mm_setcsr(_mm_getcsr() | 0x8040)
+    #else
+        #define AVOIDDENORMALS _mm_setcsr(_mm_getcsr() | 0x8000)
+    #endif
+#else
+  #warning *** puredata.cpp: NO SSE FLAG (denormals may slow things down) ***
+  #define AVOIDDENORMALS
+#endif
+
 struct Meta 
 {
     void declare (const char* key, const char* value) {}
@@ -456,6 +492,7 @@ static t_int *faust_perform(t_int *w)
   t_faust *x = (t_faust *)(w[1]);
   int n = (int)(w[2]);
   if (!x->dsp || !x->buf) return (w+3);
+  AVOIDDENORMALS;
   if (x->xfade > 0) {
     float d = 1.0f/x->n_xfade, f = (x->xfade--)*d;
     d = d/n;
@@ -618,7 +655,7 @@ static void faust_any(t_faust *x, t_symbol *s, int argc, t_atom *argv)
 	  pd_error(x, "[faust] %s: bad control argument: %s",
 		   x->label->c_str(), label);
       }
-    if (count == 0 && strcmp(label, "active") == 0)
+    if (count == 0 && strcmp(label, "active") == 0) {
       if (argc == 0) {
 	t_atom arg;
 	SETFLOAT(&arg, (float)x->active);
@@ -630,6 +667,7 @@ static void faust_any(t_faust *x, t_symbol *s, int argc, t_atom *argv)
 	x->active = (int)f;
 	x->xfade = x->n_xfade;
       }
+    }
   }
 }
 
@@ -678,8 +716,8 @@ static void *faust_new(t_symbol *s, int argc, t_atom *argv)
     x->outputs = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
     x->buf = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
   }
-  if (x->n_in > 0 && x->inputs == NULL ||
-      x->n_out > 0 && (x->outputs == NULL || x->buf == NULL))
+  if ((x->n_in > 0 && x->inputs == NULL) ||
+      (x->n_out > 0 && (x->outputs == NULL || x->buf == NULL)))
     goto error;
   for (int i = 0; i < x->n_out; i++)
     x->buf[i] = NULL;
