@@ -1,3 +1,25 @@
+/************************************************************************
+ ************************************************************************
+    FAUST Architecture File
+	Copyright (C) 2008-2011 Kjetil Matheussen <k.s.matheussen@notam02.no>
+    ---------------------------------------------------------------------
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU Lesser General Public License as 
+	published by the Free Software Foundation; either version 2.1 of the 
+	License, or (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU Lesser General Public License for more details.
+
+    You should have received a copy of the GNU Lesser General Public
+ 	License along with the GNU C Library; if not, write to the Free
+  	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+  	02111-1307 USA. 
+ ************************************************************************
+ ************************************************************************/
+
 /* link with  */
 #include <stdlib.h>
 #include <stdio.h>
@@ -434,14 +456,16 @@ GTKUI::GTKUI(char * name, int* pargc, char*** pargv)
 
 void GTKUI::pushBox(int mode, GtkWidget* w)
 {
-	assert(++fTop < stackSize);
+	++fTop;
+	assert(fTop < stackSize);
 	fMode[fTop] 	= mode;
 	fBox[fTop] 		= w;
 }
 
 void GTKUI::closeBox()
 {
-	assert(--fTop >= 0);
+	--fTop;
+	assert(fTop >= 0);
 }
 
 
