@@ -1,3 +1,6 @@
+// If other than 'faust2sc --prefix Faust' is used, sed this as well:
+#define SC_FAUST_PREFIX "Faust"
+
 //-------------------------------------------------------------------
 // FAUST architecture file for SuperCollider.
 // Copyright (C) 2005-2008 Stefan Kersten.
@@ -370,6 +373,9 @@ static size_t unitSize();
 // Convert a file name to a valid unit name.
 static std::string fileNameToUnitName(const std::string& fileName);
 
+// Convert the XML unit name to a valid class name.
+static std::string normalizeClassName(const std::string& name);
+
 void initState(const std::string& name, int sampleRate)
 {
     g_unitName = strdup(name.c_str());
@@ -406,6 +412,24 @@ std::string fileNameToUnitName(const std::string& fileName)
 
 static InterfaceTable *ft;
 
+// The SuperCollider UGen class name generated here must match
+// that generated by faust2sc:
+static std::string normalizeClassName(const std::string& name)
+{
+  std::string s;
+  char c;
+
+  unsigned int i=0;
+  bool upnext=true;
+  while (c=name[i++]) {
+    if (upnext) { c = toupper(c); upnext=false; }
+    if ( (c == '_') || (c == '-') || isspace(c)) { upnext=true; continue; }
+    s += c;
+    if (i > 31) { break; }
+  }
+  return s;
+}
+
 extern "C"
 {
     void load(InterfaceTable*);
@@ -558,16 +582,20 @@ void Faust_Dtor(Faust* unit)  // module destructor
 
 FAUST_EXPORT void load(InterfaceTable* inTable)
 {
+
     ft = inTable;
 
     Meta meta;
     mydsp::metadata(&meta);
-    std::string name(meta["name"]);
+
+    std::string name = meta["name"];
 
     if (name.empty()) {
         name = fileNameToUnitName(__FILE__);
     }
 
+    name = normalizeClassName(name);
+
     if (name.empty()) {
         // Catch empty name
         Print("*** Faust: supercollider.cpp: "
@@ -576,6 +604,10 @@ FAUST_EXPORT void load(InterfaceTable* inTable)
         return;
     }
 
+    if (strncmp(name.c_str(),SC_FAUST_PREFIX,strlen(SC_FAUST_PREFIX))!=0){
+      name = SC_FAUST_PREFIX + name;
+    }
+
     // Initialize global data
     // TODO: Use correct sample rate
     initState(name, 48000);
