@@ -59,9 +59,6 @@ extern map<string, string>		gDocMathStringMap;
 
 extern bool		getSigListNickName(Tree t, Tree& id);
 
-static void		extractMetadata(const string& fulllabel, string& label, map<string, set<string> >& metadata);
-static string	rmWhiteSpaces(const string& s);
-
 static const unsigned int MAX_RIGHT_MEMBER	= 20;
 static const unsigned int MAX_SUB_EXPR		= 10;
 
@@ -282,7 +279,7 @@ string DocCompiler::generateNumber (Tree sig, const string& exp)
 
 	// check for number occuring in delays
 	if (o->getMaxDelay()>0) {
-		getTypedNames(getSigType(sig), "r", ctype, vname);
+		getTypedNames(getCertifiedSigType(sig), "r", ctype, vname);
 		gDocNoticeFlagMap["recursigs"] = true;
 		//cerr << "- r : generateNumber : \"" << vname << "\"" << endl;            
 		generateDelayVec(sig, exp, ctype, vname, o->getMaxDelay());
@@ -301,7 +298,7 @@ string DocCompiler::generateFConst (Tree sig, const string& file, const string&
     Occurences* o = fOccMarkup.retrieve(sig);
 
     if (o->getMaxDelay()>0) {
-        getTypedNames(getSigType(sig), "r", ctype, vname);
+        getTypedNames(getCertifiedSigType(sig), "r", ctype, vname);
 		gDocNoticeFlagMap["recursigs"] = true;
 		//cerr << "- r : generateFConst : \"" << vname << "\"" << endl;            
         generateDelayVec(sig, exp, ctype, vname, o->getMaxDelay());
@@ -326,7 +323,7 @@ string DocCompiler::generateFVar (Tree sig, const string& file, const string& ex
     Occurences* o = fOccMarkup.retrieve(sig);
 
     if (o->getMaxDelay()>0) {
-        getTypedNames(getSigType(sig), "r", ctype, vname);
+        getTypedNames(getCertifiedSigType(sig), "r", ctype, vname);
 		gDocNoticeFlagMap["recursigs"] = true;
 		//cerr << "- r : generateFVar : \"" << vname << "\"" << endl;            
 		setVectorNameProperty(sig, vname);
@@ -381,6 +378,10 @@ string DocCompiler::generateOutput (Tree sig, const string& idx, const string& a
 
 /**
  * Generate binary operations, managing priority parenthesis.
+ * ((a*b)+c) can be written (a*b+c) if priority(*) > priority(+)
+ * ((a*b)*c) can be writteb (a*b*c) if * is associative
+ * Associative operation should have a distinc priority from other operations.
+ * Non associative operations can share the same priority.
  *
  * @param	sig			The signal expression to treat.
  * @param	opcode		The operation code, as described in gBinOpLateqTable.
@@ -395,6 +396,12 @@ string DocCompiler::generateOutput (Tree sig, const string& idx, const string& a
  * @todo	Handle integer arithmetics, by testing arguments type,
  * and printing dedicated operators (\oplus, \odot, \ominus, \oslash).
  */
+
+/// associative operations are + * | & xor
+static bool associative (int opcode) {
+    return (opcode == kAdd) || (opcode == kMul) || (opcode == kAND) || (opcode == kOR) || (opcode == kXOR);
+}
+
 string DocCompiler::generateBinOp(Tree sig, int opcode, Tree arg1, Tree arg2, int priority)
 {
 	string s;
@@ -403,13 +410,14 @@ string DocCompiler::generateBinOp(Tree sig, int opcode, Tree arg1, Tree arg2, in
 	/* Priority parenthesis handling. */
 	string lpar = "";
 	string rpar = "";
-	if (thisPriority < priority) {
-		lpar = " \\left(";
-		rpar = "\\right) ";
-	}
+    if ( (thisPriority < priority) || ((thisPriority == priority) && !associative(opcode)) ) {
+        // (a+b)*c or (a/b)/c need parenthesis
+        lpar = " \\left(";
+        rpar = "\\right) ";
+    }
 	
-	Type t1 = getSigType(arg1);
-	Type t2 = getSigType(arg2);
+	Type t1 = getCertifiedSigType(arg1);
+	Type t2 = getCertifiedSigType(arg2);
 	bool intOpDetected = false;
 	if ( (t1->nature() == kInt) && (t2->nature() == kInt) ) {
 		intOpDetected = true;
@@ -539,7 +547,7 @@ string DocCompiler::generateCacheCode(Tree sig, const string& exp)
         if (getVectorNameProperty(sig, vectorname)) {
 			return exp;
 		}
-        getTypedNames(getSigType(sig), "r", ctype, vname);
+        getTypedNames(getCertifiedSigType(sig), "r", ctype, vname);
 		gDocNoticeFlagMap["recursigs"] = true;
 		//cerr << "- r : generateCacheCode : vame=\"" << vname << "\", for sig=\"" << ppsig(sig) << "\"" << endl;
         if (sharing>1) {
@@ -570,7 +578,7 @@ string DocCompiler::generateCacheCode(Tree sig, const string& exp)
 string DocCompiler::generateVariableStore(Tree sig, const string& exp)
 {
     string      vname, ctype;
-    Type        t = getSigType(sig);
+    Type        t = getCertifiedSigType(sig);
 	
     switch (t->variability()) {
 			
@@ -683,7 +691,7 @@ string DocCompiler::generateVBargraph(Tree sig, Tree path, Tree min, Tree max, c
 {
 	string varname = getFreshID("{u_g}");
 
-	Type t = getSigType(sig);
+	Type t = getCertifiedSigType(sig);
 	switch (t->variability()) {
 
 		case kKonst :
@@ -703,7 +711,7 @@ string DocCompiler::generateHBargraph(Tree sig, Tree path, Tree min, Tree max, c
 {
 	string varname = getFreshID("{u_g}");
 
-	Type t = getSigType(sig);
+	Type t = getCertifiedSigType(sig);
 	switch (t->variability()) {
 
 		case kKonst :
@@ -761,7 +769,7 @@ string DocCompiler::generateDocConstantTbl (Tree /*tbl*/, Tree size, Tree isig)
     }
 
     // allocate a name v_i for the table
-    getTypedNames(getSigType(isig), "v", ctype, vname);
+    getTypedNames(getCertifiedSigType(isig), "v", ctype, vname);
 	
     // add a comment on tables in the notice
 		gDocNoticeFlagMap["tablesigs"] = true;
@@ -826,7 +834,7 @@ string DocCompiler::generateDocWriteTbl (Tree /*tbl*/, Tree size, Tree isig, Tre
 
 
     // allocate a name w_i for the table
-    getTypedNames(getSigType(isig), "w", ctype, vname);
+    getTypedNames(getCertifiedSigType(isig), "w", ctype, vname);
 
     // add a comment on tables in the notice
     gDocNoticeFlagMap["tablesigs"] = true;
@@ -916,7 +924,7 @@ void DocCompiler::generateRec(Tree sig, Tree var, Tree le, int priority)
             // this projection is used
             used[i] = true;
 			//cerr << "generateRec : used[" << i << "] = true" << endl;            
-            getTypedNames(getSigType(e), "r", ctype[i],  vname[i]);
+            getTypedNames(getCertifiedSigType(e), "r", ctype[i],  vname[i]);
 			gDocNoticeFlagMap["recursigs"] = true;
 			//cerr << "- r : generateRec setVectorNameProperty : \"" << vname[i] << "\"" << endl;
 			setVectorNameProperty(e, vname[i]);
@@ -1064,7 +1072,7 @@ string DocCompiler::generateXtended 	(Tree sig, int priority)
 
 	for (int i=0; i<sig->arity(); i++) {
 		args.push_back(CS(sig->branch(i), 0));
-		types.push_back(getSigType(sig->branch(i)));
+		types.push_back(getCertifiedSigType(sig->branch(i)));
 	}
 
 	if (p->needCache()) {
@@ -1170,7 +1178,7 @@ string DocCompiler::generateFixDelay (Tree sig, Tree exp, Tree delay, int priori
 string DocCompiler::generateDelayVec(Tree sig, const string& exp, const string& ctype, const string& vname, int mxd)
 {
 	string s = generateDelayVecNoTemp(sig, exp, ctype, vname, mxd);
-	if (getSigType(sig)->variability() < kSamp) {
+	if (getCertifiedSigType(sig)->variability() < kSamp) {
         return exp;
 	} else {
 		return s;
@@ -1339,118 +1347,3 @@ void DocCompiler::getUIDocInfos(Tree path, string& label, string& unit)
 }
 
 
-/**
- * Extracts metadata from a UI label : 'vol [unit: dB]' -> 'vol' + metadata map.
- */
-static void extractMetadata(const string& fulllabel, string& label, map<string, set<string> >& metadata)
-{
-    enum {kLabel, kEscape1, kEscape2, kEscape3, kKey, kValue};
-    int state = kLabel; int deep = 0;
-    string key, value;
-	
-    for (unsigned int i=0; i < fulllabel.size(); i++) {
-        char c = fulllabel[i];
-        switch (state) {
-            case kLabel :
-                assert (deep == 0);
-                switch (c) {
-                    case '\\' : state = kEscape1; break;
-                    case '[' : state = kKey; deep++; break;
-                    default : label += c;
-                }
-                break;
-				
-            case kEscape1 :
-                label += c;
-                state = kLabel;
-                break;
-				
-            case kEscape2 :
-                key += c;
-                state = kKey;
-                break;
-				
-            case kEscape3 :
-                value += c;
-                state = kValue;
-                break;
-				
-            case kKey :
-                assert (deep > 0);
-                switch (c) {
-                    case '\\' :  state = kEscape2;
-						break;
-						
-                    case '[' :  deep++;
-						key += c;
-						break;
-						
-                    case ':' :  if (deep == 1) {
-						state = kValue;
-					} else {
-						key += c;
-					}
-						break;
-                    case ']' :  deep--;
-						if (deep < 1) {
-							metadata[rmWhiteSpaces(key)].insert("");
-							state = kLabel;
-							key="";
-							value="";
-						} else {
-							key += c;
-						}
-						break;
-                    default :   key += c;
-                }
-                break;
-				
-            case kValue :
-                assert (deep > 0);
-                switch (c) {
-                    case '\\' : state = kEscape3;
-						break;
-						
-                    case '[' :  deep++;
-						value += c;
-						break;
-						
-                    case ']' :  deep--;
-						if (deep < 1) {
-							metadata[rmWhiteSpaces(key)].insert(rmWhiteSpaces(value));
-							state = kLabel;
-							key="";
-							value="";
-						} else {
-							value += c;
-						}
-						break;
-                    default :   value += c;
-                }
-                break;
-				
-            default :
-                cerr << "ERROR unrecognized state (in extractMetadata) : " << state << endl;
-        }
-    }
-    label = rmWhiteSpaces(label);
-}
-
-
-/**
- * rmWhiteSpaces(): Remove the leading and trailing white spaces of a string
- * (but not those in the middle of the string)
- */
-static string rmWhiteSpaces(const string& s)
-{
-    size_t i = s.find_first_not_of(" \t");
-    size_t j = s.find_last_not_of(" \t");
-	
-    if ( (i != string::npos) & (j != string::npos) ) {
-        return s.substr(i, 1+j-i);
-    } else {
-        return "";
-    }
-}
-
-
