@@ -34,12 +34,13 @@ class PSDev : public device
 		PSDev(const char*,double,double);
 		~PSDev();
 		void rect(double, double, double, double, const char*, const char*);
-		void rond(double,double,double);
+        void triangle(double x,double y,double l,double h, const char* color, const char* link, bool leftright);
+        void rond(double,double,double);
 		void carre(double,double,double);
 		void fleche(double,double,double,int);
 		void trait(double,double,double,double);
 		void dasharray(double,double,double,double);
-		void text(double,double,const char*);
+        void text(double,double,const char*, const char* link);
 		void label(double,double,const char*);
 		void markSens(double,double,int);
 		void Error(const char*,const char*,int,double,double,double);
