@@ -57,6 +57,9 @@
 #include "drawschema.hh"
 #include "compatibility.hh"
 #include "names.hh"
+#include "description.hh"
+#include "property.hh"
+
 
 
 #if 0
@@ -114,6 +117,7 @@
 #define uicolor "#477881"
 #define slotcolor "#47945E"
 #define numcolor "#f44800"
+#define invcolor "#ffffff"
 #endif
 
 using namespace std;
@@ -243,10 +247,16 @@ static void writeSchemaFile(Tree bd)
 		SVGDev dev(s1.str().c_str(), ts->width(), ts->height());
 		ts->place(0,0, kLeftRight);
 		ts->draw(dev);
+        { collector c; ts->collectTraits(c); c.draw(dev); }
 	} else {
 		PSDev dev(s1.str().c_str(), ts->width(), ts->height());
 		ts->place(0,0, kLeftRight);
 		ts->draw(dev);
+        {
+            collector c;
+            ts->collectTraits(c);
+            c.draw(dev);
+        }
 	}
 }
 
@@ -315,6 +325,66 @@ static char* legalFileName(Tree t, int n, char* dst)
 
 //------------------------ generating the schema -------------------------
 
+
+/**
+ * isInverter(t) returns true if t == '*(-1)'. This test is used
+ * to simplify diagram by using a special symbol for inverters.
+ */
+Tree gInverter[6];
+
+static bool isInverter(Tree t)
+{
+    // init gInverted table. For some reason doesn't work if done outside
+    if (gInverter[0] == 0) {
+        gInverter[0] = boxSeq(boxPar(boxWire(), boxInt(-1)),boxPrim2(sigMul));
+        gInverter[1] = boxSeq(boxPar(boxInt(-1), boxWire()),boxPrim2(sigMul));
+        gInverter[2] = boxSeq(boxPar(boxWire(), boxReal(-1.0)),boxPrim2(sigMul));
+        gInverter[3] = boxSeq(boxPar(boxReal(-1.0), boxWire()),boxPrim2(sigMul));
+        gInverter[4] = boxSeq(boxPar(boxInt(0), boxWire()),boxPrim2(sigSub));
+        gInverter[5] = boxSeq(boxPar(boxReal(0.0), boxWire()),boxPrim2(sigSub));
+    };
+
+    //cerr << "isInverter " << t << '$' << boxpp(t) << endl;
+    for (int i=0; i<6; i++) {
+        if (t == gInverter[i]) return true;
+    }
+    return false;
+}
+
+
+/**
+ * Compute the Pure Routing property, that is expressions
+ * only made of cut, wires and slots. No labels will be
+ * dispayed for pure routing expressions.
+ */
+property<bool> gPureRoutingProperty;
+
+static bool isPureRouting(Tree t)
+{
+    bool    r;
+    int     ID;
+    Tree    x,y;
+
+    if (gPureRoutingProperty.get(t,r)) {
+        return r;
+    } else if (    isBoxCut(t)
+                || isBoxWire(t)
+                || isInverter(t)
+                || isBoxSlot(t, &ID)
+                || (isBoxPar(t,x,y) && isPureRouting(x) && isPureRouting(y))
+                || (isBoxSeq(t,x,y) && isPureRouting(x) && isPureRouting(y))
+                || (isBoxSplit(t,x,y) && isPureRouting(x) && isPureRouting(y))
+                || (isBoxMerge(t,x,y) && isPureRouting(x) && isPureRouting(y))
+              ) {
+        gPureRoutingProperty.set(t,true);
+        return true;
+    } else {
+        gPureRoutingProperty.set(t,false);
+        return false;
+    }
+}
+
+
 /**
  * Generate an appropriate schema according to
  * the type of block diagram. When folding is requiered,
@@ -343,7 +413,7 @@ static schema* generateDiagramSchema(Tree t)
 		scheduleDrawing(t);
 		return makeBlockSchema(ins, outs, s.str(), linkcolor, l.str());
 
-	} else  if (getDefNameProperty(t, id) && ! isBoxSlot(t)) {
+    } else  if (getDefNameProperty(t, id) && ! isPureRouting(t)) {
 		// named case : not a slot, with a name
 		// draw a line around the object with its name
 		stringstream 	s; s << tree2str(id);
@@ -378,6 +448,8 @@ static schema* generateInsideSchema(Tree t)
 
 	if (xt)							{ return makeBlockSchema(xt->arity(), 1, xt->name(), normalcolor, ""); }
 
+    else if (isInverter(t))         { return makeInverterSchema(invcolor); }
+
 	else if (isBoxInt(t, &i))		{ stringstream 	s; s << i; return makeBlockSchema(0, 1, s.str(), numcolor, "" ); }
 	else if (isBoxReal(t, &r)) 		{ stringstream 	s; s << r; return makeBlockSchema(0, 1, s.str(), numcolor, "" ); }
 	else if (isBoxWire(t)) 			{ return makeCableSchema(); }
@@ -403,13 +475,13 @@ static schema* generateInsideSchema(Tree t)
 	else if (isBoxHBargraph(t))		{ return generateBargraphSchema(t); }
 
 	// don't draw group rectangle when labels are empty (ie "")
-	else if (isBoxVGroup(t,l,a))	{ 	stringstream s; s << "vgroup(" << tree2str(l) << ")";
+    else if (isBoxVGroup(t,l,a))	{ 	stringstream s; s << "vgroup(" << extractName(l) << ")";
 										schema* r = generateDiagramSchema(a);
 									  	return makeDecorateSchema(r, 10, s.str()); }
-	else if (isBoxHGroup(t,l,a))	{ 	stringstream s; s << "hgroup(" << tree2str(l) << ")";
+    else if (isBoxHGroup(t,l,a))	{ 	stringstream s; s << "hgroup(" << extractName(l) << ")";
 										schema* r = generateDiagramSchema(a);
 									  	return makeDecorateSchema(r, 10, s.str()); }
-	else if (isBoxTGroup(t,l,a))	{ 	stringstream s; s << "tgroup(" << tree2str(l) << ")";
+    else if (isBoxTGroup(t,l,a))	{ 	stringstream s; s << "tgroup(" << extractName(l) << ")";
 										schema* r = generateDiagramSchema(a);
 									  	return makeDecorateSchema(r, 10, s.str()); }
 
@@ -437,7 +509,67 @@ static schema* generateInsideSchema(Tree t)
 	}
 }
 
-//------------------------ specific schema -------------------------
+/**
+ * Convert User interface element into a textual representation
+ */
+static void UserInterfaceDescription(Tree box, string& d)
+{
+    Tree    t1, label, cur, min, max, step;
+    stringstream 	fout;
+    // user interface
+         if (isBoxButton(box, label))	fout << "button(" << extractName(label) << ')';
+    else if (isBoxCheckbox(box, label))	fout << "checkbox(" << extractName(label) << ')';
+    else if (isBoxVSlider(box, label, cur, min, max, step)) 	{
+        fout << "vslider("
+             << extractName(label) << ", "
+             << boxpp(cur) << ", "
+             << boxpp(min) << ", "
+             << boxpp(max) << ", "
+             << boxpp(step)<< ')';
+    }
+    else if (isBoxHSlider(box, label, cur, min, max, step)) 	{
+        fout << "hslider("
+             << extractName(label) << ", "
+             << boxpp(cur) << ", "
+             << boxpp(min) << ", "
+             << boxpp(max) << ", "
+             << boxpp(step)<< ')';
+    }
+    else if (isBoxVGroup(box, label, t1)) {
+        fout << "vgroup(" << extractName(label) << ", " << boxpp(t1, 0) << ')';
+    }
+    else if (isBoxHGroup(box, label, t1)) {
+        fout << "hgroup(" << extractName(label) << ", " << boxpp(t1, 0) << ')';
+    }
+    else if (isBoxTGroup(box, label, t1)) {
+        fout << "tgroup(" << extractName(label) << ", " << boxpp(t1, 0) << ')';
+    }
+    else if (isBoxHBargraph(box, label, min, max)) 	{
+        fout << "hbargraph("
+             << extractName(label) << ", "
+             << boxpp(min) << ", "
+             << boxpp(max) << ')';
+    }
+    else if (isBoxVBargraph(box, label, min, max)) 	{
+        fout << "vbargraph("
+             << extractName(label) << ", "
+             << boxpp(min) << ", "
+             << boxpp(max) << ')';
+    }
+    else if (isBoxNumEntry(box, label, cur, min, max, step)) 	{
+        fout << "nentry("
+             << extractName(label) << ", "
+             << boxpp(cur) << ", "
+             << boxpp(min) << ", "
+             << boxpp(max) << ", "
+             << boxpp(step)<< ')';
+    }
+    else {
+        cerr << "INTERNAL ERROR : unknow user interface element " << endl;
+        exit(0);
+    }
+    d = fout.str();
+}
 
 
 /**
@@ -445,23 +577,18 @@ static schema* generateInsideSchema(Tree t)
  */
 static schema* 	generateUserInterfaceSchema(Tree t)
 {
-	stringstream 	s;
-	s << boxpp(t);
-
-	return makeBlockSchema(0, 1, s.str(), uicolor, "");
+    string s; UserInterfaceDescription(t,s);
+    return makeBlockSchema(0, 1, s, uicolor, "");
 }
 
 
-
 /**
  * Generate a 1->1 block schema for a user interface bargraph
  */
 static schema* generateBargraphSchema(Tree t)
 {
-	stringstream 	s;
-	s << boxpp(t);
-
-	return makeBlockSchema(1, 1, s.str(), uicolor, "");
+    string s; UserInterfaceDescription(t,s);
+    return makeBlockSchema(1, 1, s, uicolor, "");
 }
 
 
