@@ -3,6 +3,8 @@
 #include <math.h>
 #include "sigtyperules.hh"
 
+#include "floats.hh"
+
 class MinPrim : public xtended
 {
 
@@ -33,24 +35,24 @@ class MinPrim : public xtended
 	
 	virtual Tree	computeSigOutput (const vector<Tree>& args) 
 	{
-		float f,g; int i,j;
+		double f,g; int i,j;
 		
 		assert (args.size() == arity());
 		
-		if (isFloat(args[0]->node(),&f)) {
+		if (isDouble(args[0]->node(),&f)) {
 		
-			if (isFloat(args[1]->node(), &g)) {
+			if (isDouble(args[1]->node(), &g)) {
 				return tree(min(f, g));
 			} else if (isInt(args[1]->node(),&j)) {
-				return tree(min(f, float(j)));
+				return tree(min(f, double(j)));
 			} else {
 				return tree(symbol(), args[0], args[1]);
 			}
 				
 		} else if (isInt(args[0]->node(),&i)) {
 		
-			if (isFloat(args[1]->node(), &g)) {
-				return tree(min(float(i), g));
+			if (isDouble(args[1]->node(), &g)) {
+				return tree(min(double(i), g));
 			} else if (isInt(args[1]->node(),&j)) {
 				return tree(min(i, j));
 			} else {
@@ -76,6 +78,15 @@ class MinPrim : public xtended
 		} 			
 	}
 	
+	virtual string 	generateLateq (Lateq* lateq, const vector<string>& args, const vector<Type>& types)
+	{
+		assert (args.size() == arity());
+		assert (types.size() == arity());
+		
+		Type t = infereSigType(types);
+		return subst("\\min\\left( $0, $1 \\right)", args[0], args[1]); 			
+	}
+
 };
 
 
