@@ -3,7 +3,9 @@
 #include "Text.hh"
 #include <math.h>
 
-#ifdef WIN32 
+#include "floats.hh"
+
+#if defined(WIN32) && ! defined(__MINGW32__) 
 /* missing on Windows : see http://bugs.mysql.com/bug.php?id=15936 */
 inline double rint(double nr)
 {
@@ -47,7 +49,7 @@ class RintPrim : public xtended
 		num n;
 		assert (args.size() == arity());
 		if (isNum(args[0],n)) {
-			return tree(rintf(float(n)));
+			return tree(rint(double(n)));
 		} else {
 			return tree(symbol(), args[0]);
 		}
@@ -56,7 +58,17 @@ class RintPrim : public xtended
 	virtual string 	generateCode (Klass* klass, const vector<string>& args, const vector<Type>& types)
 	{
 		assert (args.size() == arity());
-		return subst("rintf($0)", args[0]);
+		assert (types.size() == arity());
+
+		return subst("rint$1($0)", args[0], isuffix());
+	}
+	
+	virtual string 	generateLateq (Lateq* lateq, const vector<string>& args, const vector<Type>& types)
+	{
+		assert (args.size() == arity());
+		assert (types.size() == arity());
+        
+		return subst("\\left[ {$0} \\right]", args[0]);
 	}
 	
 };
