@@ -18,7 +18,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  ************************************************************************
  ************************************************************************/
-#define FAUSTVERSION "0.9.43"
+#define FAUSTVERSION "0.9.46"
 
 #include <stdio.h>
 #include <string.h>
@@ -147,7 +147,7 @@ bool			gGroupTaskSwitch= false;
 bool            gUIMacroSwitch  = false;
 bool            gDumpNorm       = false;
 
-int             gTimeout        = 0;            // time out to abort compiler
+int             gTimeout        = 600;            // time out to abort compiler (in seconds)
 
 int             gFloatSize = 1;
 
@@ -360,7 +360,7 @@ bool process_cmdline(int argc, char* argv[])
 void printversion()
 {
 	cout << "FAUST, DSP to C++ compiler, Version " << FAUSTVERSION << "\n";
-	cout << "Copyright (C) 2002-2011, GRAME - Centre National de Creation Musicale. All rights reserved. \n\n";
+	cout << "Copyright (C) 2002-2012, GRAME - Centre National de Creation Musicale. All rights reserved. \n\n";
 }
 
 
