@@ -22,6 +22,7 @@
  
  
 #include "enrobage.hh"
+#include <vector>
 #include <string>
 #include <limits.h>
 #include <stdlib.h>
@@ -32,16 +33,94 @@ extern string gFaustSuperSuperDirectory;
 extern string gFaustSuperDirectory;
 extern string gFaustDirectory;
 extern string gMasterDirectory;
+extern string gClassName;
+
 //----------------------------------------------------------------
 
 
+/**
+ * Returns true is a line is blank (contains only white caracters)
+ */
+static bool isBlank(const string& s) {
+    for (size_t i=0; i<s.size(); i++) {
+        if (s[i] != ' ' && s[i] != '\t') return false;
+    }
+    return true;
+}
+
+
+/**
+ * Replace every occurrence of oldstr by newstr inside str. str is modified
+ * and returned as reference for convenience
+ */
+static string& replaceOccurences(string& str, const string& oldstr, const string& newstr)
+{
+    string::size_type l1 = oldstr.length();
+    string::size_type l2 = newstr.length();
+
+    string::size_type pos = str.find(oldstr);
+    while ( pos != string::npos) {
+        str.replace(pos, l1, newstr);
+        pos = str.find(oldstr, pos + l2);
+    }
+    return str;
+}
+
+
+/**
+ * Used when copying architecture files to replace default mydsp
+ * class name with the user specified one
+ */
+static string& replaceClassName(string& str)
+{
+    return replaceOccurences(str, "mydsp", gClassName);
+}
+
+
+/**
+ * Copy or remove license header. Architecture files can contain a header specifying
+ * the license. If this header contains an exception tag (for example "FAUST COMPILER EXCEPTION")
+ * it is an indication for the compiler to remove the license header from the resulting code. 
+ * A header is the first non blank line that begins a comment.
+ */
+void streamCopyLicense(istream& src, ostream& dst, const string& exceptiontag)
+{
+    string          s;
+    vector<string>	H;
+
+    // skip blank lines
+    while (getline(src,s) && isBlank(s)) dst << s << endl;
+
+    // first non blank should start a comment
+    if (s.find("/*")==string::npos) { dst << s << endl; return; }
+
+    // copy the header into H
+    bool remove = false;
+    H.push_back(s);
+
+    while (getline(src,s) && s.find("*/")==string::npos) {
+        H.push_back(s);
+        if (s.find(exceptiontag) != string::npos) remove=true;
+    }
+
+    // copy the header unless explicitely granted to remove it
+    if (!remove) {
+        // copy the header
+        for (unsigned int i=0; i<H.size(); i++) {
+            dst << H[i] << endl;
+        }
+        dst << s << endl;
+    }
+}
+
+
 /**
  * Copy src to dst until specific line.
  */
 void streamCopyUntil(istream& src, ostream& dst, const string& until)
-{ 
-	string	s;
-	while ( getline(src,s) && (s != until) ) dst << s << endl;
+{
+    string	s;
+    while ( getline(src,s) && (s != until) ) dst << replaceClassName(s) << endl;
 }
 
 /**
@@ -50,7 +129,7 @@ void streamCopyUntil(istream& src, ostream& dst, const string& until)
 void streamCopy(istream& src, ostream& dst)
 { 
 	string	s;
-	while ( getline(src,s)) dst << s << endl;
+    while ( getline(src,s)) dst << replaceClassName(s) << endl;
 }
 
 /**
@@ -59,7 +138,7 @@ void streamCopy(istream& src, ostream& dst)
 void streamCopyUntilEnd(istream& src, ostream& dst)
 { 
 	string	s;
-	while ( getline(src,s) ) dst << s << endl;
+    while ( getline(src,s) ) dst << replaceClassName(s) << endl;
 }
 
 
@@ -76,6 +155,15 @@ ifstream* open_arch_stream(const char* filename)
 	    ifstream* f = new ifstream();
 	    f->open(filename, ifstream::in); if (f->is_open()) return f; else delete f;
     }
+    char *envpath = getenv("FAUST_LIB_PATH");
+    if (envpath!=NULL) {
+		if (chdir(envpath)==0) {
+			ifstream* f = new ifstream();
+			f->open(filename, ifstream::in);
+			if (f->is_open()) return f; else delete f;
+		}
+    }
+	err = chdir(old);
 	if ( (chdir(gFaustDirectory.c_str())==0) && (chdir("architecture")==0) ) {
 		//cout << "enrobage.cpp : 'architecture' directory found in gFaustDirectory" << endl;
         ifstream* f = new ifstream();
@@ -243,6 +331,7 @@ static void buildFullPathname(string& fullpath, const char* filename)
 FILE* fopensearch(const char* filename, string& fullpath)
 {   
     FILE* f;
+    char* envpath;
 
     if ((f = fopen(filename, "r"))) { 
     	buildFullPathname(fullpath, filename); 
@@ -251,6 +340,9 @@ FILE* fopensearch(const char* filename, string& fullpath)
     if ((f = fopenat(fullpath, gMasterDirectory, filename))) { 
     	return f;
     }
+    if ((envpath = getenv("FAUST_LIB_PATH")) && (f = fopenat(fullpath, envpath, filename))) {
+        return f;
+    }
     if ((f = fopenat(fullpath, gFaustDirectory, "architecture", filename))) { 
     	return f;
     }
@@ -266,7 +358,7 @@ FILE* fopensearch(const char* filename, string& fullpath)
 FILE* fopensearch(const char* filename, string& fullpath)
 {   
     FILE* f;
-
+    char* envpath;
 
     if ((f = fopen(filename, "r"))) { 
     	buildFullPathname(fullpath, filename); 
@@ -275,6 +367,9 @@ FILE* fopensearch(const char* filename, string& fullpath)
     if ((f = fopenat(fullpath, gMasterDirectory, filename))) { 
     	return f;
     }
+    if ((envpath = getenv("FAUST_LIB_PATH")) && (f = fopenat(fullpath, envpath, filename))) {
+		return f;
+    }
     if ((f = fopenat(fullpath, gFaustDirectory, "architecture", filename))) { 
     	return f;
     }
@@ -326,6 +421,10 @@ FILE* fopensearch(const char* filename, string& fullpath)
 (((ch) == DIR_SEPARATOR) || ((ch) == DIR_SEPARATOR_2))
 #endif /* DIR_SEPARATOR_2 */
 
+
+/**
+ * returns a pointer on the basename part of name
+ */
 const char* filebasename(const char* name)
 {
 #if defined (HAVE_DOS_BASED_FILE_SYSTEM)
@@ -345,3 +444,27 @@ const char* filebasename(const char* name)
 	return base;
 }
 
+
+/**
+ * returns a string containing the dirname of name
+ * If no dirname, returns "."
+ */
+string filedirname(const string& name)
+{
+    const char*         base = filebasename(name.c_str());
+    const unsigned int  size = base-name.c_str();
+    string              dirname;
+
+    if (size==0) {
+        dirname += '.';
+
+    } else if (size==1) {
+        dirname += name[0];
+
+    } else {
+        for (unsigned int i=0; i<size-1; i++) {
+            dirname += name[i];
+        }
+    }
+    return dirname;
+}
