@@ -8,6 +8,7 @@
 #include <iostream>
 #include <map>
 #include <list>
+#include <string>
 
 
 #include "sourcereader.hh"
@@ -16,6 +17,11 @@
 
 using namespace std;
 
+extern map<Tree, set<Tree> > gMetaDataSet;
+extern string gMasterDocument;
+extern vector<Tree> gDocVector;
+extern bool gLatexDocSwitch;
+
 /****************************************************************
  						Parser variables
 *****************************************************************/
@@ -131,7 +137,7 @@ static Tree makeDefinition(list<Tree>& variants)
 
 /**
  * Formats a list of raw definitions represented by triplets
- * <name,arglit,body> into abstractions or pattern 
+ * <name,arglist,body> into abstractions or pattern 
  * matching rules when appropriate.
  * 
  * @param rldef list of raw definitions in reverse order
@@ -179,10 +185,12 @@ Tree formatDefinitions(Tree rldef)
 
 Tree SourceReader::parse(string fname)
 {
+	string	fullpath;
+	
 	yyerr = 0;
 	
 	yyfilename = fname.c_str();
-	yyin = fopensearch(yyfilename);
+	yyin = fopensearch(yyfilename, fullpath);
 	if (yyin == NULL) {
 		fprintf(stderr, "ERROR : Unable to open file  %s \n", yyfilename); 
 		exit(1);
@@ -198,6 +206,8 @@ Tree SourceReader::parse(string fname)
 		exit(1);
 	}
 
+	// we have parsed a valid file
+	fFilePathnames.push_back(fullpath);
 	return gResult;
 }
 
@@ -232,6 +242,25 @@ Tree SourceReader::getlist(string fname)
 }
 
  
+/**
+ * Return a vector of pathnames representing the list 
+ * of all the source files that have been required
+ * to evaluate process (those in fFileCache)
+ */
+
+vector<string> SourceReader::listSrcFiles()
+{
+//	vector<string> 						srcfiles;
+
+//	for (map<string, Tree>::const_iterator p = fFileCache.begin(); p != fFileCache.end(); p++) {
+//		srcfiles.push_back(p->first);
+//	}
+
+//	return srcfiles;	
+	return fFilePathnames;
+}
+
+ 
 /**
  * Return the list of definitions where all imports have been expanded.
  * 
@@ -270,4 +299,24 @@ Tree SourceReader::expandrec(Tree ldef, set<string>& visited, Tree lresult)
 	return lresult;
 }
 				
-			
+
+void declareMetadata(Tree key, Tree value)
+{
+    if (gMasterDocument == yyfilename) {
+        // inside master document, no prefix needed to declare metadata
+        gMetaDataSet[key].insert(value);
+    } else {
+        string fkey(yyfilename);
+        fkey += "/";
+        fkey += tree2str(key);
+        gMetaDataSet[tree(fkey.c_str())].insert(value);
+    }
+    //cout << "Master " << gMasterDocument  << ", file " << yyfilename <<  " : declare " << *key << "," << *value << endl;
+}
+
+
+void declareDoc(Tree t)
+{
+	//gLatexDocSwitch = true;
+	gDocVector.push_back(t);
+}
