@@ -142,9 +142,9 @@ static int infereSigOrder(Tree sig)
 		
 	else if (isSigAttach(sig, s1, s2)) 			return O(s1);
 				
-	else if (isRec(sig, var, body))				exit(1); //return 3;	// not supposed to appen.
+	else if (isRec(sig, var, body))				exit(1); //return 3;  // not supposed to happen.
 				
-	else if (isRef(sig, var))					exit(1); //return 3;  // not supposed to appen. 
+	else if (isRef(sig, var))					exit(1); //return 3;  // not supposed to happen. 
 
 	else if (isProj(sig, &i, s1))				return 3;
 	                                                	
@@ -158,7 +158,7 @@ static int infereSigOrder(Tree sig)
 
     else if (isSigDocWriteTbl(sig,s1,s2,s3,s4)) return 3;
 
-    else if (isSigDocAccessTbl(sig,s1,s2))         return 3;
+    else if (isSigDocAccessTbl(sig,s1,s2))      return 3;
 
     else if (isSigGen(sig, s1)) 				return 3;
 		
